var fs = require('fs'),
    merge = require('deepmerge'),
    Utils = require('../../utils');

module.exports = {
    readSwagger: function readSwagger() {
        var _ = this;

        try {
            return JSON.parse(
                fs.readFileSync(_.swagger.path)
            );
        } catch (e) {
            return {};
        }
    },

    saveSwagger: function saveSwagger(done) {
        var _ = this;

        _.logger.log('Swagger saving...');
        fs.writeFileSync(_.swagger.path, JSON.stringify(_.swagger.data, null, 4));
        _.logger.log('Swagger saved.');
        typeof done === 'function' && done();
    },

    generateSwagger: function generateSwagger(options, done) {
        if (!options && !done || typeof options === 'function') {
            done = options;
            options = {};
        }

        var _ = this,
            template = _.handlebars.compile(_.swagger.templates.resource),
            path, route, method, obj;

        _.logger.log('Swagger generating...');

        if (options.regeneratePaths) {
            _.swagger.data.paths = {};
        }

        for (path in _.routes) {
            route = _.routes[path];

            for (method in _.routes[path]) {
                try {
                    obj = JSON.parse(template({
                        operationId: Utils.prepPath(path.replace(/^\//, '') + '-' + method.toUpperCase(), {
                            prefix: _.sdk.name,
                            humanize: true
                        }),
                        sdkMethod: Utils.prepPath(path, {
                            camelCase: true
                        }),
                        path: path,
                        method: method,
                        handler: _.routes[path][method],
                        defaults: _.defaults
                    }));
                } catch (e) {
                    _.logger.error(e);
                    obj = {};
                }

                _.swagger.data = merge(obj, _.swagger.data);
            }
        }

        _.logger.log('Swagger generated.');

        _.saveSwagger();

        typeof done === 'function' && done(null, _.swagger.data);

        return _.swagger.data;
    }
};
